#!/usr/bin/python

import os
import time
import re
import sys

SNAIL_MAXSIZE=500
class UserError(Exception):
    """User defined exception."""
    def __init__(self, value):
        """Simple constructor with a value (string)."""
        self.value = value
    def __str__(self):
        """Wrapper to handle display of Exception."""
        return repr(self.value)

class Snail:
    """Utility class used to return i,j coordinates arround
    a given start point."""

    def __init__(self):
	"""Constructor sets position to 0,0."""
	self.i=0
	self.j=0
	self.n_total=0 # total distance done by our "turtle"
	self.n_partial=0 # partial distance (in one direction)
	self.dir=0 # 0=North, 1=West, 2=South, 3=East
	self.step=1
    
    def get_next(self):
	"""Returns the next point coordinates."""
	self.check_maxsize()
	coords=(self.i,self.j)
	if self.dir==0:
	    self.i-=1
	elif self.dir==1:
	    self.j+=1
	elif self.dir==2:
	    self.i+=1
	elif self.dir==3:
	    self.j-=1
	self.n_partial+=1
	if (self.n_partial>=self.step):
	    if (self.dir % 2):
		self.step+=1
	    self.n_partial=0
            self.dir=(self.dir+1)%4
	self.n_total+=1
	return coords    

    def check_maxsize(self): 
	"""Checks that we don't exceed maxsize."""
	if (self.i<=-SNAIL_MAXSIZE or 
	    self.j<=-SNAIL_MAXSIZE or 
	    self.i>=SNAIL_MAXSIZE or 
	    self.j>=SNAIL_MAXSIZE):
	    raise UserError("Indices out of range (%d,%d)" % (i,j))

class Crawler:
    """Abstract class which contains utility method to retrieve URLs."""

    def __init__(self):
	"""Various wizardry."""
	self.snail=Snail()

    def call_curl(self,url,params={},filename=None):
	"""Call curls as a command line to get an URL.
	If filename is defined, stores the resulting page
	in a file, else returns an array of lines."""
	cmd="curl \"%s\"" % url
	for key in params.keys():
	    cmd+=" -d %s=\"%s\"" % (key,params[key])
	if filename:
	    cmd+=" -o %s" % filename
	process=os.popen(cmd)
	content=process.readlines()
	time.sleep(1) # be nice with server, give it a rest 8-)
	return content

    def get_basename(self,i,j):
        """Returns the base name of the image, used in the
        image creation form."""
        return "i%03dj%03d" % (i+SNAIL_MAXSIZE,j+SNAIL_MAXSIZE)
    
    def get_tiffname(self,i,j):
        """Returns the name of the file which will be used
        when storing the image on the filesystem."""
        return "%s.tiff.Z" % self.get_basename(i,j)

    def run(self,lat,long,number):
	"""Runs the crawler, requires a get_image function
	to be defined, along with lat_step and long_step
	(not defined here, this is abstract..."""
	for n in xrange(number):
	    (i,j)=self.snail.get_next()
	    tiffname=self.get_tiffname(i,j)
	    if (os.path.exists(tiffname)):
		print "File %s already exists" % tiffname
	    else:
		image_lat=lat+i*self.lat_step
		image_long=long+j*self.long_step
		print "Getting file %s (%0.5f,%0.5f)" % (tiffname,
							 image_lat,
							 image_long)
		self.get_image(image_lat,image_long,i,j)


class RasterRoam(Crawler):
    """This class is designed to get "orthophotos" from the
    Raster Roam web site http://geoengine.nima.mil/. Use it
    at your own risk since it can generate heavy downloads
    and stress the server. Try not to overload the server
    and have a reasonable and nice attitude."""    

    def __init__(self):
	"""Constructor sets up constants."""
	Crawler.__init__(self)
	self.filename_mask="i%03dj%03d"
	self.bitmap_width=640
	self.bitmap_height=480
	self.lat_step=-0.040
	self.long_step=0.060

    def step1_calc_inputpath(self,lat,long):
	"""Fires a get request to get an HTTP page which contains
	the "inputpath" parameter in a hidden form field. This
	"inputpath" cannot be guest by using lat and long without
	calling the webserver."""
        regex=re.compile("name=\"inputpath\"\s+value=\"(.*)\"",re.I)
        url="http://geoengine.nima.mil/muse-cgi-bin/rast_roam.cgi?cenlat=%0.6f&cenlon%0.6f&coordtype=dd&product_level=DOI+10M" % (lat,long)
        self.inputpath=""
        for line in self.call_curl(url):
            m=regex.search(line)
            if m:
                self.inputpath=m.groups()[0]
        if not self.inputpath:
            raise UserError("Unable to get inputpath")            

    def step2_generate_image(self,lat,long,i,j):
        """Requests the image generation on the server, which
        will return a page which contains a link to the
        downloadable .Z file."""
        url="http://geoengine.nima.mil/muse-cgi-bin/rast_imp2.cgi"
        params={'inputpath':self.inputpath,
                'exttype': '1', 
                'intptype': '1',
                'product': 'DOI',
                'data_level': '10M',
                'relfile_output': 'y',
                'centerlat': lat,
                'centerlon': long,
                'compress_type': '2',
                'from_rast_roam': 'yes',
                'enhntype': '1',
                'rpfscale': '10M',
                'pngclass': ' ',
                'pngrelease': ' ',
                'outtype':'5',
                'outputfile':self.get_basename(i,j),
                'imagex':self.bitmap_width,
                'imagey':self.bitmap_height,
                'Import':'Center Point Download'}
        #self.call_curl(url,params,"generate_image.html")
        self.call_curl(url,params)

    def step3_download_image(self,i,j):
        """Downloads the previously generated image."""
        url="http://geoengine.nima.mil/geospatial/SW_TOOLS/NIMAMUSE/webinter/muse_webinter_output/%s.tif.Z" % self.get_basename(i,j)
        self.call_curl(url,{},self.get_tiffname(i,j))
        pass

    def step4_delete_image(self,i,j):
        """Removing the image from the server. This is important
        for raster roam is freely available and we shouldn't pollute
        and fill the server with the images generated when harvesting."""
        url="http://geoengine.nima.mil/muse-cgi-bin/delete_download.cgi"
        params={'rm':'rm -rf',
                'file': "/data/html/geospatial/SW_TOOLS/NIMAMUSE/webinter/muse_webinter_output/%s.tif.Z" % self.get_basename(i,j), 
                'prevhtml': "/geospatial/SW_TOOLS/NIMAMUSE/webinter/rast_roam.html",
                'prevtitle': 'NGA Raster Roam',
                'submit': 'Delete Download File'}        
        #self.call_curl(url,params,"delete.html")
        self.call_curl(url,params)
        pass

    def get_image(self,lat,long,i,j):
	"""Gets a georeferenced .tiff image from the server."""
        try:
            self.step1_calc_inputpath(lat,long)
            self.step2_generate_image(lat,long,i,j)
            self.step3_download_image(i,j)
            self.step4_delete_image(i,j)
        except UserError, e:
            print "User error: ", e
            if re.compile("inputpath").search(e.value):
                # The error is probably "no data" here, so
                # we create a blank file which will prevent
                # later tries. Deleting zero size files
                # isn't hard anyways.
                f=open(self.get_tiffname(i,j),"w")
                f.close()
	pass


rr=RasterRoam()		

rr.run(48.866666,2.333333,320*240)
       

	


